/* 
 *  Copyright (C) 1999-2001 Bernd Gehrmann
 *                          bernd@physik.hu-berlin.de
 *
 * This program may be distributed under the terms of the Q Public
 * License as defined by Trolltech AS of Norway and appearing in the
 * file LICENSE.QPL included in the packaging of this file.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */


#include <stdlib.h>
#include <unistd.h>
#include <qaction.h>
#include <qcstring.h>
#include <qdir.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qsplitter.h>
#include <qtextstream.h>
#include <qtooltip.h>
#include <qvaluelist.h>
#include <kaccel.h>
#include <kaction.h>
#include <kapp.h>
#include <kconfig.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kkeydialog.h>
#include <klocale.h>
#include <kmenubar.h>
#include <ktoolbarbutton.h>
#include <kprocess.h>
#include <kfileitem.h>
#include <kdebug.h>

#include "logdlg.h"
#include "loglist.h"
#include "diffdlg.h"
#include "resolvedlg.h"
#include "annotatedlg.h"
#include "commitdlg.h"
#include "updatedlg.h"
#include "checkoutdlg.h"
#include "tagdlg.h"
#include "mergedlg.h"
#include "historydlg.h"
#include "updateview.h"
#include "protocolview.h"
#include "cvsprogressdlg.h"
#include "repositorydlg.h"
#include "settingsdlg.h"
#include "changelogdlg.h"
#include "misc.h"
#include "../config.h"

#include "toplevel.h"
#include "toplevel.moc"


#define COMMIT_SPLIT_CHAR '\r'

TopLevel::TopLevel(const char *name)
    : KMainWindow(0, name )
{
    KConfig *config = kapp->config();
    config->setGroup("LookAndFeel");
    bool splitHorz = config->readBoolEntry("SplitHorizontally");

    splitter = new QSplitter(splitHorz? QSplitter::Horizontal : QSplitter::Vertical, this);
 
    update = new UpdateView(splitter);
    update->setFocusPolicy(StrongFocus);
    update->setFocus();

    //    connect( update, SIGNAL(rightButtonPressed(QListViewItem*, const QPoint&, int)),
    //             this, SLOT(popupRequested()) );
    connect( update, SIGNAL(contextMenu()),
             this, SLOT(popupRequested()) );
    connect( update, SIGNAL(fileOpened(QString)),
             this, SLOT(openFile(QString)) );

    protocol = new ProtocolView(splitter);
    protocol->setFocusPolicy(StrongFocus);

    setCentralWidget(splitter);
    setupAccel();
    setupMenuBar();
    setupToolBar();
    setupStatusBar();
    setupToolTips();
    updateAccel();
}


TopLevel::~TopLevel()
{
    delete file;
    delete view;
    delete settings;
    delete help;
}


void TopLevel::setupAccel()
{
    accel = new KAccel(this);

    accel->insertItem( i18n("Open File"),
                       "Open", CTRL+Key_O );
    accel->insertItem( i18n("Update Files"),
                       "Update", CTRL+Key_U );
    accel->insertItem( i18n("Status"),
                       "Status", Key_F5 );
    accel->insertItem( i18n("Commit file"),
                       "Commit", Key_NumberSign );
    accel->insertItem( i18n("Add File to Repository"),
                       "Add", Key_Plus );
    accel->insertItem( i18n("Remove File from Repository"),
                       "Remove", Key_Minus );
    accel->insertItem( i18n("Browse log file"),
                       "Log", CTRL+Key_L );
    accel->insertItem( i18n("View Annotate"),
                       "Annotate", CTRL+Key_A );
    accel->insertItem( i18n("Diff against repository"),
                       "Diff", CTRL+Key_D );
    accel->insertItem( i18n("Hide all files"),
                       "Hide all files", CTRL+Key_F );
    accel->insertItem( i18n("Hide up-to-date files"),
                       "Hide up-to-date files", CTRL+Key_T );
    
    accel->connectItem( "Open",     this, SLOT(slotOpen()) );
    accel->connectItem( "Update",   this, SLOT(slotUpdate()) );
    accel->connectItem( "Status",   this, SLOT(slotStatus()) );
    accel->connectItem( "Commit",   this, SLOT(slotCommit()) );
    accel->connectItem( "Add",      this, SLOT(slotAdd()) );
    accel->connectItem( "Remove",   this, SLOT(slotRemove()) );
    accel->connectItem( "Log",      this, SLOT(slotBrowseLog()) );
    accel->connectItem( "Annotate", this, SLOT(slotAnnotate()) );
    accel->connectItem( "Diff",     this, SLOT(slotDiff()) );
    accel->connectItem( "Hide all files",        this, SLOT(slotHideFiles()) );
    accel->connectItem( "Hide up-to-date files", this, SLOT(slotHideUpToDate()) );
    accel->connectItem( KStdAccel::Quit, this, SLOT(slotExit()) );

    accel->readSettings();
}


void TopLevel::updateAccel()
{
    accel->changeMenuAccel( file, file_open,   "Open" );
    accel->changeMenuAccel( file, file_update, "Update" );
    accel->changeMenuAccel( file, file_status, "Status" );
    accel->changeMenuAccel( file, file_commit, "Commit" );
    accel->changeMenuAccel( file, file_add,    "Add" );
    accel->changeMenuAccel( file, file_remove, "Remove" );
    accel->changeMenuAccel( view, view_log,    "Log" );
    accel->changeMenuAccel( view, view_anno,   "Annotate" );
    accel->changeMenuAccel( view, view_diff,   "Diff" );
    accel->changeMenuAccel( settings, settings_hidefiles,    "Hide all files" );
    accel->changeMenuAccel( settings, settings_hideuptodate, "Hide up-to-date files");
    accel->changeMenuAccel( file, file_exit,   KStdAccel::Quit );
}


void TopLevel::setupMenuBar()
{
    //
    // File menu
    //
    file = new QPopupMenu();

    file_sandbox = file->insertItem( i18n("O&pen Sandbox..."),
                                     this, SLOT(slotOpenSandbox()) );
    file_recent  = file->insertItem( i18n("Op&en recent..."),
                                     recent = new QPopupMenu() );
    file_chlog   = file->insertItem( i18n("&Insert ChangeLog entry..."),
                                     this, SLOT(slotChangeLog()) );
    
    file->insertSeparator();
    
    file_update = file->insertItem( i18n("&Update"),
                                    this, SLOT(slotUpdate()) );
    file_status = file->insertItem( i18n("&Status"),
                                    this, SLOT(slotStatus()) );

    file->insertSeparator();
    
    file_open    = file->insertItem( BarIcon("fileopen"), i18n("&Open..."),
                                     this, SLOT(slotOpen()) );
    file_resolv  = file->insertItem( i18n("Reso&lve..."),
                                     this, SLOT(slotResolve()) );

    file->insertSeparator();
    
    file_commit = file->insertItem( i18n("&Commit..."),
                                    this, SLOT(slotCommit()) );
    file_add    = file->insertItem( i18n("&Add to Repository..."),
                                    this, SLOT(slotAdd()) );
    file_addbin = file->insertItem( i18n("Add &binary..."),
                                    this, SLOT(slotAddBinary()) );
    file_remove = file->insertItem( i18n("&Remove from Repository..."),
                                    this, SLOT(slotRemove()) );
    file_revert = file->insertItem( i18n("Rever&t"),
                                    this, SLOT(slotRevert()) );

    file->insertSeparator();

    file_exit   = file->insertItem( BarIcon("exit"), i18n("&Quit"),
                                    this, SLOT(slotExit()) );

    //
    // View menu
    //
    view = new QPopupMenu();
    
    view_log    = view->insertItem( i18n("Browse &Log..."),
                                    this, SLOT(slotBrowseLog()) );
#if 0
    view_mlog   = view->insertItem( i18n("Browse Multi-File Log..."),
                                    this, SLOT(slotBrowseMultiLog()) );
#endif
    view_anno   = view->insertItem( i18n("&Annotate..."),
                                    this, SLOT(slotAnnotate()) );
    view_diff   = view->insertItem( i18n("&Difference to Repository..."),
                                    this, SLOT(slotDiff()) );
    view_change = view->insertItem( i18n("Last &Change..."),
                                    this, SLOT(slotLastChange()) );
    view_history= view->insertItem( i18n("&History..."),
                                    this, SLOT(slotHistory()) );
    

    view->insertSeparator();

    view_unfold = view->insertItem( i18n("&Unfold file tree"),
                                    update, SLOT(unfoldTree()) );
    view_fold =   view->insertItem( i18n("&Fold file tree"),
                                    update, SLOT(foldTree()) );

    //
    // Advanced menu
    //
    advanced = new QPopupMenu();

    advanced_createtag = advanced->insertItem( i18n("&Tag/Branch..."),
                                               this, SLOT(slotCreateTag()) );
    advanced_deltag    = advanced->insertItem( i18n("&Delete Tag..."),
                                               this, SLOT(slotDeleteTag()) );
    advanced_uptag     = advanced->insertItem( i18n("&Update to Tag/Date..."),
                                               this, SLOT(slotUpdateToTag()) );
    advanced_uphead    = advanced->insertItem( i18n("Update to &HEAD"),
                                               this, SLOT(slotUpdateToHead()) );
    advanced_merge     = advanced->insertItem( i18n("&Merge..."),
                                               this, SLOT(slotMerge()) );

    advanced->insertSeparator();

    advanced_addwatch  = advanced->insertItem( i18n("&Add watch..."),
                                               this, SLOT(slotAddWatch()) );
    advanced_remwatch  = advanced->insertItem( i18n("&Remove watch..."),
                                               this, SLOT(slotRemoveWatch()) );
    advanced_watchers  = advanced->insertItem( i18n("Show &watchers"),
                                               this, SLOT(slotShowWatchers()) );
    
    advanced->insertSeparator();

    advanced_edit      = advanced->insertItem( i18n("Ed&it files"),
                                               this, SLOT(slotEdit()) );
    advanced_unedit    = advanced->insertItem( i18n("U&nedit files"),
                                               this, SLOT(slotUnedit()) );
    advanced_editors   = advanced->insertItem( i18n("Show &editors"),
                                               this, SLOT(slotShowEditors()) );
    
    advanced->insertSeparator();

    advanced_lock      = advanced->insertItem( i18n("&Lock files"),
                                               this, SLOT(slotLock()) );
    advanced_unlock    = advanced->insertItem( i18n("Unl&ock files"),
                                               this, SLOT(slotUnlock()) );
    
    advanced->insertSeparator();

    advanced_patch     = advanced->insertItem( i18n("Create &patch against repository"),
                                               this, SLOT(slotMakePatch()) );

    //
    // Repository menu
    //
    repo = new QPopupMenu();

    repo_checkout = repo->insertItem( i18n("&Checkout..."),
                                        this, SLOT(slotCheckout()) );
    repo_import   = repo->insertItem( i18n("&Import..."),
                                      this, SLOT(slotImport()) );

    repo->insertSeparator();
    
    repo_repos    = repo->insertItem( i18n("&Repositories..."),
                                      this, SLOT(slotRepositories()) );

    //
    // Settings menu
    //
    settings = new QPopupMenu();

    settings_hidefiles   = settings->insertItem( i18n("Hide all &files"),
                                                 this, SLOT(slotHideFiles()) );
    settings_hideuptodate= settings->insertItem( i18n("Hide up-&to-date files"),
                                                 this, SLOT(slotHideUpToDate()) );
    settings_hideremoved = settings->insertItem( i18n("Hide removed files"),
                                                 this, SLOT(slotHideRemoved()) );

    settings->insertSeparator();
    
    settings_create      = settings->insertItem( i18n("Create &directories on update"),
                                                 this, SLOT(slotCreateDirs()) );
    settings_prune       = settings->insertItem( i18n("&Prune empty directories on update"),
                                                 this, SLOT(slotPruneDirs()) );
    settings_updaterecur = settings->insertItem( i18n("&Update recursively"),
                                                 this, SLOT(slotUpdateRecursive()) );
    settings_commitrecur = settings->insertItem( i18n("C&ommit and remove recursively"),
                                                 this, SLOT(slotCommitRecursive()) );
    settings_docvsedit   = settings->insertItem( i18n("Do cvs &edit automatically when necessary"),
                                                 this, SLOT(slotDoCVSEdit()) );
    
    settings->insertSeparator();
    
    settings_configure   = settings->insertItem( i18n("&Configure Cervisia..."),
                                                 this, SLOT(slotConfigure()) );
    settings_keys        = settings->insertItem( i18n("Configure &keybindings..."),
                                                 this, SLOT(slotConfigureKeys()) );
    
    help = new QPopupMenu();
    
    help_contents = help->insertItem( i18n("&Contents"),
                                      this, SLOT(slotHelp()) );
    
    help->insertSeparator();
    
    help_cvsinfo  = help->insertItem( i18n("CVS &Info"),
                                      this, SLOT(slotCVSInfo()) );
    help_about    = help->insertItem( BarIcon("cervisia"), i18n("&About Cervisia"),
                                      this, SLOT(aboutCervisia()) );
    
    KMenuBar *menubar = menuBar();
    menubar->insertItem( i18n("&File"), file );
    menubar->insertItem( i18n("&View"), view );
    menubar->insertItem( i18n("&Advanced"), advanced );
    menubar->insertItem( i18n("&Repository"), repo );
    menubar->insertItem( i18n("&Settings"), settings );
    menubar->insertSeparator();
    menubar->insertItem( i18n("&Help"), help );

    connect( file, SIGNAL(aboutToShow()), SLOT(updateMenu()) );
    connect( view, SIGNAL(aboutToShow()), SLOT(updateMenu()) );
    connect( advanced, SIGNAL(aboutToShow()), SLOT(updateMenu()) );
    connect( recent, SIGNAL(aboutToShow()), SLOT(fillRecentPopup()) );
    connect( recent, SIGNAL(activated(int)), SLOT(recentActivated(int)) );

    connect( file, SIGNAL(highlighted(int)), SLOT(showMenuToolTip(int)) );
    connect( file, SIGNAL(aboutToHide()), statusBar(), SLOT(clear()) );
    connect( view, SIGNAL(highlighted(int)), SLOT(showMenuToolTip(int)) );
    connect( view, SIGNAL(aboutToHide()), statusBar(), SLOT(clear()) );
    connect( advanced, SIGNAL(highlighted(int)), SLOT(showMenuToolTip(int)) );
    connect( advanced, SIGNAL(aboutToHide()), statusBar(), SLOT(clear()) );
    connect( repo, SIGNAL(highlighted(int)), SLOT(showMenuToolTip(int)) );
    connect( repo, SIGNAL(aboutToHide()), statusBar(), SLOT(clear()) );
    connect( settings, SIGNAL(highlighted(int)), SLOT(showMenuToolTip(int)) );
    connect( settings, SIGNAL(aboutToHide()), statusBar(), SLOT(clear()) );
    connect( help, SIGNAL(highlighted(int)), SLOT(showMenuToolTip(int)) );
    connect( help, SIGNAL(aboutToHide()), statusBar(), SLOT(clear()) );
}


void TopLevel::setupToolBar()
{
    KToolBar *toolbar = toolBar();

    toolbar->insertButton( BarIcon("up"), 101, SIGNAL(clicked()),
                           this, SLOT(slotCommit()),
                           true, i18n("Commit") );

    toolbar->insertButton( BarIcon("down"), 102, SIGNAL(clicked()),
                           this, SLOT(slotUpdate()),
                           true, i18n("Update") );

    toolbar->insertSeparator();
    
    toolbar->insertButton( BarIcon("fileopen"), 103, SIGNAL(clicked()),
                           this, SLOT(slotOpen()),
                           true, i18n("Open file") );
    
    toolbar->insertButton( BarIcon("help"), 104, SIGNAL(clicked()),
                           this, SLOT(slotHelp()),
                           true, i18n("Help") );

    toolbar->insertSeparator();
    
    toolbar->insertButton( BarIcon("stop"), 42, SIGNAL(clicked()),
                           protocol, SLOT(cancelJob()),
                           false, i18n("Stop") );
}


void TopLevel::setupStatusBar()
{
    filterLabel = new QLabel("UR", statusBar());
    filterLabel->setFixedSize(filterLabel->sizeHint());
    filterLabel->setText("");
    QToolTip::add(filterLabel, i18n("F - All files are hidden, the tree shows only directories\n"
                                    "N - All up-to-date files are hidden\n"
                                    "R - All removed files are hidden"));
    statusBar()->addWidget(filterLabel, 0, true);
}


void TopLevel::setupToolTips()
{
    tooltips.insert(file_sandbox,         i18n("Opens a CVS working directory in the main window")); 
    tooltips.insert(file_recent,          i18n("Opens a recently used CVS working directory"));
    tooltips.insert(file_chlog,           i18n("Opens the ChangeLog file in the sandbox for editing"));
    tooltips.insert(file_update,          i18n("Updates (cvs update) the selected files and directories"));
    tooltips.insert(file_status,          i18n("Updates the status (cvs -n update) of the selected files and directories"));
    tooltips.insert(file_open,            i18n("Opens the marked file for editing"));
    tooltips.insert(file_resolv,          i18n("Opens the resolve dialog with the selected file"));
    tooltips.insert(file_commit,          i18n("Commits the selected files"));
    tooltips.insert(file_add,             i18n("Adds (cvs add) the selected files to the repository"));
    tooltips.insert(file_addbin,          i18n("Adds (cvs -kb add) the selected files as binaries to the repository"));
    tooltips.insert(file_remove,          i18n("Removes (cvs remove) the selected files from the repository"));
    tooltips.insert(file_revert,          i18n("Reverts (cvs update -C) the selected files (only cvs 1.11)"));
    tooltips.insert(file_exit,            i18n("Quits Cervisia"));
    tooltips.insert(view_log,             i18n("Shows the revision tree of the selected file"));
    tooltips.insert(view_anno,            i18n("Shows a blame-annotated view of the selected file"));
    tooltips.insert(view_diff,            i18n("Shows the differences of the selected file to the BASE version"));
    tooltips.insert(view_change,          i18n("Shows the differences between the last two revisions of the selected file"));
    tooltips.insert(view_history,         i18n("Shows the CVS history as reported by the server"));
    tooltips.insert(view_unfold,          i18n("Opens all branches of the file tree"));
    tooltips.insert(view_fold,            i18n("Closes all branches of the file tree"));
    tooltips.insert(advanced_createtag,   i18n("Creates a tag or branch for the selected files"));
    tooltips.insert(advanced_deltag,      i18n("Deletes a tag from the selected files"));
    tooltips.insert(advanced_uptag,       i18n("Updates the selected files to a given tag, branch or date"));
    tooltips.insert(advanced_uphead,      i18n("Updates the selected files to the HEAD revision"));
    tooltips.insert(advanced_merge,       i18n("Merges a branch or a set of modifications into the selected files"));
    tooltips.insert(advanced_addwatch,    i18n("Adds a watch for the selected files"));
    tooltips.insert(advanced_remwatch,    i18n("Removes a watch from the selected files"));
    tooltips.insert(advanced_watchers,    i18n("Shows the watchers of the selected files"));
    tooltips.insert(advanced_edit,        i18n("Edits (cvs edit) the selected files")); 
    tooltips.insert(advanced_unedit,      i18n("Unedits (cvs unedit) the selected files"));
    tooltips.insert(advanced_editors,     i18n("Shows the editors of the selected files"));
    tooltips.insert(advanced_lock,        i18n("Locks the selected files, so that others can't modify them"));
    tooltips.insert(advanced_unlock,      i18n("Unlocks the selected files"));
    tooltips.insert(advanced_patch,       i18n("Creates a patch from the modifications in your sandbox"));
    tooltips.insert(repo_checkout,        i18n("Allows you to checkout a module from a repository"));
    tooltips.insert(repo_import,          i18n("Allows you to import a module into a repository"));
    tooltips.insert(repo_repos,           i18n("Configures a list of repositories you regularly use"));
    tooltips.insert(settings_hidefiles,   i18n("Determines whether only directories are shown"));
    tooltips.insert(settings_hideuptodate,i18n("Determines whether up-to-date files are hidden"));
    tooltips.insert(settings_hideremoved, i18n("Determines whether removed files are hidden"));
    tooltips.insert(settings_create,      i18n("Determines whether updates create directories"));
    tooltips.insert(settings_prune,       i18n("Determines whether updates remove empty directories"));
    tooltips.insert(settings_updaterecur, i18n("Determines whether updates are recursive"));
    tooltips.insert(settings_commitrecur, i18n("Determines whether commits and removes are recursive"));
    tooltips.insert(settings_docvsedit,   i18n("Determines whether automatical cvs edit is active"));
    tooltips.insert(settings_configure,   i18n("Configures Cervisia"));
    tooltips.insert(settings_keys,        i18n("Configures key bindings"));
    tooltips.insert(help_contents,        i18n("Opens the help browser with Cervisia's documentation"));
    tooltips.insert(help_cvsinfo,         i18n("Opens the help browser with the CVS documentation"));
    tooltips.insert(help_about,           i18n("About Cervisia"));
}


void TopLevel::popupRequested()
{
    QPopupMenu pop;
    int id_open   = pop.insertItem( i18n("&Open..."),
                                    this, SLOT(slotOpen()) );
    int id_resolv = pop.insertItem( i18n("Reso&lve..."),
                                    this, SLOT(slotResolve()) );
    int id_commit = pop.insertItem( i18n("&Commit..."),
                                    this, SLOT(slotCommit()) );
    int id_add    = pop.insertItem( i18n("&Add to Repository..."),
                                    this, SLOT(slotAdd()) );
    int id_remove = pop.insertItem( i18n("&Remove from Repository..."),
                                    this, SLOT(slotRemove()) );
    (void) id_open;
    
    bool single = update->hasSingleSelection();
    pop.setItemEnabled(id_resolv, single);
    bool nojob = !toolBar()->getButton(42)->isEnabled();
    pop.setItemEnabled(id_commit, nojob);
    pop.setItemEnabled(id_add, nojob);
    pop.setItemEnabled(id_remove, nojob);
    pop.exec(QCursor::pos());
}


void TopLevel::updateMenu()
{
    bool single = update->hasSingleSelection();
    toolBar()->getButton(103)->setEnabled(single);

    file->setItemEnabled(file_resolv, single);
    view->setItemEnabled(view_log, single);
    view->setItemEnabled(view_anno, single);
    view->setItemEnabled(view_diff, single);
    view->setItemEnabled(view_change, single);

    bool nojob = !toolBar()->getButton(42)->isEnabled();
    toolBar()->getButton(101)->setEnabled(nojob);
    toolBar()->getButton(102)->setEnabled(nojob);
    file->setItemEnabled(file_update, nojob);
    file->setItemEnabled(file_status, nojob);
    file->setItemEnabled(file_commit, nojob);
    file->setItemEnabled(file_add, nojob);
    file->setItemEnabled(file_addbin, nojob);
    file->setItemEnabled(file_remove, nojob);
    file->setItemEnabled(file_revert, nojob);
    advanced->setItemEnabled(advanced_createtag, nojob);
    advanced->setItemEnabled(advanced_deltag, nojob);
    advanced->setItemEnabled(advanced_uptag, nojob);
    advanced->setItemEnabled(advanced_uphead, nojob);
    advanced->setItemEnabled(advanced_merge, nojob);
    advanced->setItemEnabled(advanced_addwatch, nojob);
    advanced->setItemEnabled(advanced_remwatch, nojob);
    advanced->setItemEnabled(advanced_watchers, nojob);
    advanced->setItemEnabled(advanced_edit, nojob);
    advanced->setItemEnabled(advanced_unedit, nojob);
    advanced->setItemEnabled(advanced_editors, nojob);
    advanced->setItemEnabled(advanced_lock, nojob);
    advanced->setItemEnabled(advanced_unlock, nojob);
    repo->setItemEnabled(repo_checkout, nojob);
    repo->setItemEnabled(repo_import, nojob);
}


void TopLevel::aboutCervisia()
{
    QString aboutstr(i18n("Cervisia %1\n"
                          "(Using KDE %2)\n"
                          "\n"
                          "Copyright (C) 1999-2001\n"
                          "Bernd Gehrmann <bernd@physik.hu-berlin.de>\n"
                          "\n"
                          "This program may be distributed under the terms of the Q Public\n"
                          "License as defined by Trolltech AS of Norway and appearing in the\n"
                          "file LICENSE.QPL included in the packaging of this file.\n\n"
                          "This program is distributed in the hope that it will be useful,\n"
                          "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
                          "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."));
    QMessageBox::about(0, i18n("About Cervisia"), aboutstr.arg(VERSION).arg(KDE_VERSION_STRING));
}


void TopLevel::slotOpenSandbox()
{
    QString dirname = KFileDialog::getExistingDirectory(QDir::homeDirPath(), this,
                      i18n("Open Sandbox")); 
    if (dirname.isEmpty())
        return;

    openSandbox(dirname);
}


void TopLevel::slotChangeLog()
{
    // Modal dialog
    ChangeLogDialog *l = new ChangeLogDialog();
    if (l->readFile(sandbox + "/ChangeLog"))
        {
            if (l->exec())
                changelogstr = l->message();
        }
    
    delete l;
}


void TopLevel::slotOpen()
{
    QStringList filenames = update->fileSelection();
    if (filenames.isEmpty())
        return;
    openFiles(filenames);
}


void TopLevel::openFile(QString filename)
{
    QStringList files;
    files << filename;
    openFiles(files);
}


void TopLevel::openFiles(const QStringList &filenames)
{
    // First check the cvs edit stuff
    if (opt_doCVSEdit)
        {
            CvsProgressDialog l("Edit", this);
            l.setCaption("CVS Edit");
            QString cmdline = cvsClient() + " edit ";

            bool doit = false;
            for ( QStringList::ConstIterator it = filenames.begin();
                  it != filenames.end(); ++it )
                {
                    if (!QFileInfo(*it).isWritable())
                        {
                            doit = true;
                            break;
                        }
                    
                    cmdline += " ";
                    cmdline += KShellProcess::quote(*it);
                }
            
            if (doit)
                if (!l.execCommand(sandbox, repository, cmdline, "edit"))
                    return;
        }
    
    // Now open the files by starting a new instance of kwrite...
    KConfig *config = kapp->config();
    config->setGroup("Communication");
    KShellProcess proc("/bin/sh");
    proc << config->readEntry("Editor", "kwrite");

    for ( QStringList::ConstIterator it = filenames.begin();
          it != filenames.end(); ++it )
        proc << KShellProcess::quote(*it);
    proc.start(KProcess::DontCare);
}


void TopLevel::slotResolve()
{
    QString filename;
    update->getSingleSelection(&filename);
    if (filename.isEmpty())
        return;

    // Non-modal dialog
    ResolveDialog *l = new ResolveDialog();
    if (l->parseFile(filename))
        l->show();
    else
        delete l;
}


void TopLevel::slotUpdate()
{
    updateOrStatus(false, "");
}


void TopLevel::slotStatus()
{
    updateOrStatus(true, "");
}


void TopLevel::slotUpdateToTag()
{
    UpdateDialog *l = new UpdateDialog(sandbox, repository, this);

    if (l->exec())
        {
            QString tagopt;
            if (l->byTag())
                {
                    tagopt = "-r ";
                    tagopt += l->tag();
                }
            else
                {
                    tagopt = "-D ";
                    tagopt += KShellProcess::quote(l->date());
                }
            tagopt += " ";
            updateOrStatus(false, tagopt);
        }
    delete l;
}


void TopLevel::slotUpdateToHead()
{
    updateOrStatus(false, "-A ");
}


void TopLevel::slotRevert()
{
    updateOrStatus(false, "-C ");
}


void TopLevel::slotMerge()
{
    MergeDialog *l = new MergeDialog(sandbox, repository, this);

    if (l->exec())
        {
            QString tagopt;
            if (l->byBranch())
                {
                    tagopt = "-j ";
                    tagopt += l->branch();
                }
            else
                {
                    tagopt = "-j ";
                    tagopt += l->tag1();
                    tagopt += " -j ";
                    tagopt += l->tag2();
                }
            tagopt += " ";
            updateOrStatus(false, tagopt);
        }
    delete l;
}


void TopLevel::slotCommit()
{
    commitOrAddOrRemove(CommitDialog::Commit);
}


void TopLevel::slotAdd()
{
    commitOrAddOrRemove(CommitDialog::Add);
}


void TopLevel::slotAddBinary()
{
    commitOrAddOrRemove(CommitDialog::AddBinary);
}


void TopLevel::slotRemove()
{
    commitOrAddOrRemove(CommitDialog::Remove);
}


void TopLevel::updateOrStatus(bool noact, const QString &extraopt)
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;
    
    update->prepareJob(opt_updateRecursive,
                       noact? UpdateView::UpdateNoAct : UpdateView::Update);

    QString cmdline;
    if (noact)
        cmdline = cvsClient() + " -n update ";
    else
        cmdline = cvsClient() + " update ";
    if (opt_updateRecursive)
        cmdline += "-R ";
    else
        cmdline += "-l ";
    if (opt_createDirs)
        cmdline += "-d ";
    if (opt_pruneDirs)
        cmdline += "-P ";
    cmdline += extraopt;
    cmdline += joinLine(list);
    cmdline += " 2>&1";
    
    if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(receivedLine(QString)), update, SLOT(processUpdateLine(QString)) );
            connect( protocol, SIGNAL(jobFinished(bool)), update, SLOT(finishJob(bool)) );
            connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
        }
}


void TopLevel::commitOrAddOrRemove(CommitDialog::ActionType action)
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;
    
    // modal dialog
    CommitDialog *l = new CommitDialog(action, this);
    if (action == CommitDialog::Commit)
    {
        l->setLogMessage(changelogstr);
        l->setLogHistory(sandbox, repository, recentCommits);
    }
    l->setFileList(list);
    
    if (l->exec())
    {
        QString cmdline;
        switch (action)
        {
            case CommitDialog::Commit:
            {
                QString msg = l->logMessage();
                if( !recentCommits.contains( msg ) )
                {
                    recentCommits.prepend( msg );
                    while (recentCommits.count() > 50)
                        recentCommits.remove( recentCommits.last() );

                    KConfig* config = kapp->config();
                    config->setGroup( "CommitLogs" );
                    config->writeEntry( sandbox, recentCommits, COMMIT_SPLIT_CHAR );
                }

                update->prepareJob(opt_commitRecursive, UpdateView::Commit);
                cmdline = cvsClient() + " commit ";
                if (opt_commitRecursive)
                    cmdline += "-R ";
                else
                    cmdline += "-l ";
                cmdline += "-m ";
                cmdline += KShellProcess::quote(l->logMessage());
                cmdline += " ";
            }
            break;

            case CommitDialog::Add:
                update->prepareJob(false, UpdateView::Add);
                cmdline = cvsClient() + " add ";
            break;

            case CommitDialog::AddBinary:
                update->prepareJob(false, UpdateView::Add);
                cmdline = cvsClient() + " add -kb ";
            break;

            case CommitDialog::Remove:
                update->prepareJob(opt_commitRecursive, UpdateView::Remove);
                cmdline = cvsClient() + " remove -f ";
                if (opt_commitRecursive)
                    cmdline += "-R ";
                else
                    cmdline += "-l ";
            break;
        }

        cmdline += joinLine(list);
        cmdline += " 2>&1";
            
        if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(jobFinished(bool)),
                     update, SLOT(finishJob(bool)) );
            connect( protocol, SIGNAL(jobFinished(bool)),
                     this, SLOT(slotJobFinished(bool)) );
        }
    }
    
    delete l;
}


void TopLevel::slotExit()
{
    (void) queryExit();
    kapp->quit();
}


void TopLevel::slotBrowseLog()
{
    QString filename;
    update->getSingleSelection(&filename);
    if (filename.isEmpty())
        return;

    // Non-modal dialog
    LogDialog *l = new LogDialog();
    if (l->parseCvsLog(sandbox, repository, filename))
        l->show();
    else
        delete l;
}


#if 0
void TopLevel::slotBrowseMultiLog()
{
    QStrList list = update->multipleSelection();
    if (!list.isEmpty())
        {
            // Non-modal dialog
            MultiLogDialog *l = new MultiLogDialog();
            if (l->parseCvsLog(".", list))
                l->show();
            else
                delete l;
        }
}
#endif


void TopLevel::slotAnnotate()
{
    QString filename;
    update->getSingleSelection(&filename);
    
    if (filename.isEmpty())
        return;

    // Non-modal dialog
    AnnotateDialog *l = new AnnotateDialog();
    if (l->parseCvsAnnotate(sandbox, repository, filename, ""))
        l->show();
    else
        delete l;
}


void TopLevel::slotDiff()
{
    QString filename;
    update->getSingleSelection(&filename);
    
    if (filename.isEmpty())
        return;

    // Non-modal dialog
    DiffDialog *l = new DiffDialog();
    if (l->parseCvsDiff(sandbox, repository, filename, "", ""))
        l->show();
    else
        delete l;
}


void TopLevel::slotAddWatch()
{
    addOrRemoveWatch(WatchDialog::Add);
}


void TopLevel::slotRemoveWatch()
{
    addOrRemoveWatch(WatchDialog::Remove);
}


void TopLevel::addOrRemoveWatch(WatchDialog::ActionType action)
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    WatchDialog *l = new WatchDialog(action, this);

    if (l->exec() && l->events() != WatchDialog::None)
        {
            QString cmdline = cvsClient();
            cmdline += " watch ";
            if (action == WatchDialog::Add)
                cmdline += "add ";
            else
                cmdline += "remove ";
            
            WatchDialog::Events events = l->events();
            if (events != WatchDialog::All)
                {
                    if (events & WatchDialog::Commits)
                        cmdline += "-a commit ";
                    if (events & WatchDialog::Edits)
                        cmdline += "-a edit ";
                    if (events & WatchDialog::Unedits)
                        cmdline += "-a unedit ";
                }
            
            cmdline += joinLine(list);
                    
            if (protocol->startJob(sandbox, repository, cmdline))
                {
                    showJobStart(cmdline);
                    connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
                }
        }
        
    delete l;
}


void TopLevel::slotShowWatchers()
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    QString cmdline = cvsClient();
    cmdline += " watchers ";
    cmdline += joinLine(list);
                    
    if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
        }
}


void TopLevel::slotEdit()
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    QString cmdline = cvsClient();
    cmdline += " edit ";
    cmdline += joinLine(list);
                    
    if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
        }
}


void TopLevel::slotUnedit()
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    QString cmdline = "echo y | ";
    cmdline += cvsClient();
    cmdline += " unedit ";
    cmdline += joinLine(list);
                    
    if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
        }
}


void TopLevel::slotLock()
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    QString cmdline = cvsClient();
    cmdline += " admin -l ";
    cmdline += joinLine(list);
                    
    if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
        }
}


void TopLevel::slotUnlock()
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    QString cmdline = cvsClient();
    cmdline += " admin -u ";
    cmdline += joinLine(list);
                    
    if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
        }
}


void TopLevel::slotShowEditors()
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    QString cmdline = cvsClient();
    cmdline += " editors ";
    cmdline += joinLine(list);
                    
    if (protocol->startJob(sandbox, repository, cmdline))
        {
            showJobStart(cmdline);
            connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
        }
}


void TopLevel::slotMakePatch()
{
    CvsProgressDialog l("Diff", this);
    l.setCaption("CVS Diff");

    QString cmdline = cvsClient();
    cmdline += " diff -uR";
    if (!l.execCommand(sandbox, repository, cmdline, ""))
        return;

    QString filename = KFileDialog::getSaveFileName();
    if (filename.isEmpty())
        return;

    QFile f(filename);
    if (!f.open(IO_WriteOnly))
        {
            QMessageBox::information(this, "Cervisia",
                                     i18n("Could not open file for writing."));
            return;
        }
    QTextStream t(&f);
    QCString line;
    while (l.getOneLine(&line))
        t << line << '\n';

    f.close();
}


void TopLevel::slotImport()
{
    importOrCheckout(CheckoutDialog::Import);
}


void TopLevel::slotCheckout()
{
    importOrCheckout(CheckoutDialog::Checkout);
}


void TopLevel::importOrCheckout(CheckoutDialog::ActionType action)
{
    CheckoutDialog *l = new CheckoutDialog(action, this);

    if (l->exec())
        {
            QString cmdline = "cd ";
            cmdline += l->workingDirectory();
            cmdline += " && ";
            cmdline += cvsClient();
            cmdline += " -d ";
            cmdline += l->repository();
            if (action == CheckoutDialog::Checkout)
                {
                    cmdline += " checkout -P ";
                    cmdline += l->module();
                }
            else
                {
                    cmdline += " import";
                    if (l->importBinary())
                        cmdline += " -kb";
                    QString ignore = l->ignoreFiles().stripWhiteSpace();
                    if (!ignore.isEmpty())
                        {
                            cmdline += " -I ";
                            cmdline += KShellProcess::quote(ignore);
                        }
                    cmdline += " -m ";
                    cmdline += (QString("\"") + "" + "\" "); // log message?
                    cmdline += l->module();
                    cmdline += " ";
                    cmdline += l->vendorTag();
                    cmdline += " ";
                    cmdline += l->releaseTag();
                }
            
            if (protocol->startJob(sandbox, repository, cmdline))
                {
                    showJobStart(cmdline);
                    connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
                }
        }
        
    delete l;
}


void TopLevel::slotRepositories()
{
    RepositoryDialog *l = new RepositoryDialog(this);
    l->show();
}


void TopLevel::slotCreateTag()
{
    createOrDeleteTag(TagDialog::Create);
}


void TopLevel::slotDeleteTag()
{
    createOrDeleteTag(TagDialog::Delete);
}


void TopLevel::createOrDeleteTag(TagDialog::ActionType action)
{
    QStringList list = update->multipleSelection();
    if (list.isEmpty())
        return;

    TagDialog *l = new TagDialog(action, sandbox, repository, this);

    if (l->exec())
        {
            QString cmdline = cvsClient();
            cmdline += " tag ";
            if (action == TagDialog::Delete)
                cmdline += "-d ";
            else if (l->branchTag())
                cmdline += "-b ";
            cmdline += l->tag();
            cmdline += " ";
            cmdline += joinLine(list);
                    
            if (protocol->startJob(sandbox, repository, cmdline))
                {
                    showJobStart(cmdline);
                    connect( protocol, SIGNAL(jobFinished(bool)), this, SLOT(slotJobFinished(bool)) );
                }
        }
        
    delete l;
}



void TopLevel::slotLastChange()
{
    QString filename, revA, revB;
    update->getSingleSelection(&filename, &revA);
    if (filename.isEmpty())
        return;
    
    int pos, lastnumber;
    bool ok;
    if ( (pos = revA.findRev('.')) == -1
         || (lastnumber=revA.right(revA.length()-pos-1).toUInt(&ok), !ok) )
        {
            QMessageBox::information(this, "Cervisia", i18n("The revision looks invalid."));
            return;
        }
    if (lastnumber == 0)
        {
            QMessageBox::information(this, "Cervisia", i18n("The is the first revision on the branch."));
            return;
        }
    revB = revA.left(pos+1);
    revB += QString::number(lastnumber-1);

    // Non-modal dialog
    DiffDialog *l = new DiffDialog();
    if (l->parseCvsDiff(sandbox, repository, filename, revB, revA))
        l->show();
    else
        delete l;
}


void TopLevel::slotHistory()
{
    // Non-modal dialog
    HistoryDialog *l = new HistoryDialog();
    if (l->parseHistory(sandbox, repository))
        l->show();
    else
        delete l;
}


void TopLevel::slotHideFiles()
{
    opt_hideFiles = !opt_hideFiles;
    setFilter();
}


void TopLevel::slotHideUpToDate()
{
    opt_hideUpToDate = !opt_hideUpToDate;
    setFilter();
}


void TopLevel::slotHideRemoved()
{
    opt_hideRemoved = !opt_hideRemoved;
    setFilter();
}


void TopLevel::slotCreateDirs()
{
    opt_createDirs = !opt_createDirs;
    settings->setItemChecked(settings_create, opt_createDirs);
}


void TopLevel::slotPruneDirs()
{
    opt_pruneDirs = !opt_pruneDirs;
    settings->setItemChecked(settings_prune, opt_pruneDirs);
}


void TopLevel::slotUpdateRecursive()
{
    opt_updateRecursive = !opt_updateRecursive;
    settings->setItemChecked(settings_updaterecur, opt_updateRecursive);
}


void TopLevel::slotCommitRecursive()
{
    opt_commitRecursive = !opt_commitRecursive;
    settings->setItemChecked(settings_commitrecur, opt_commitRecursive);
}


void TopLevel::slotDoCVSEdit()
{
    opt_doCVSEdit = !opt_doCVSEdit;
    settings->setItemChecked(settings_docvsedit, opt_doCVSEdit);
}


void TopLevel::slotConfigure()
{
    SettingsDialog *l = new SettingsDialog(this);
    l->exec();

    KConfig *config = kapp->config();
    config->setGroup("LookAndFeel");
    bool splitHorz = config->readBoolEntry("SplitHorizontally");
    splitter->setOrientation(splitHorz? QSplitter::Horizontal : QSplitter::Vertical);
}


void TopLevel::slotConfigureKeys()
{
    if (KKeyDialog::configureKeys(accel))
        updateAccel();
}


void TopLevel::slotHelp()
{
    statusBar()->message(i18n("Invoking help on Cervisia"), 2000);
    kapp->invokeHelp();
}


void TopLevel::slotCVSInfo()
{
    statusBar()->message(i18n("Invoking help on CVS"), 2000);
    KApplication::startServiceByDesktopName("khelpcenter", QString("info:/cvs/Top"));
}


void TopLevel::showJobStart(const QString &cmdline)
{
    toolBar()->setItemEnabled(42, true);
    statusBar()->message(cmdline);
    updateMenu();
}


void TopLevel::slotJobFinished(bool)
{
    toolBar()->setItemEnabled(42, false);
    statusBar()->message(i18n("Done"));
    updateMenu();
}


void TopLevel::openSandbox(const QString &dirname)
{
    QFileInfo fi1(dirname);
    QString sandboxpath = fi1.absFilePath();

    // Nice side-effect of this: Deleted sandboxes are removed
    recentFiles.remove(sandboxpath);

    QFileInfo fi2(sandboxpath + "/CVS");
    if (!fi2.exists() || !fi2.isDir())
    {
        QMessageBox::information(this, "Cervisia", i18n("This is not a CVS directory."));
        return;
    }

    recentFiles.append(sandboxpath);
    while (recentFiles.count() > 10)
        recentFiles.remove(recentFiles.fromLast());

    changelogstr = "";
    sandbox = sandboxpath;
    repository = "";

    QFile f(sandbox + "/CVS/Root");
    if (f.open(IO_ReadOnly)) 
    {
        QTextStream t(&f);
        repository = t.readLine();
    }
    f.close();
    

    setCaption(sandbox + "(" + repository + ")");
    QDir::setCurrent(sandbox);
    update->openDirectory(sandbox);

    KConfig *config = kapp->config();
    config->setGroup("General");
    bool dostatus = config->readBoolEntry(repository.contains(":")?
                                          "StatusForRemoteRepos" : "StatusForLocalRepos", false);
    if (dostatus)
    {
        update->setSelected(update->firstChild(), true);
        updateOrStatus(true, "");
    }

    //load the recentCommits for this app from the KConfig app
    config->setGroup( "CommitLogs" );
    recentCommits = config->readListEntry( sandbox, COMMIT_SPLIT_CHAR );

}


void TopLevel::fillRecentPopup()
{
    recent->clear();
    int n = 0;
    for ( QStringList::Iterator it = recentFiles.begin();
          it != recentFiles.end(); ++it )
        {
            recent->insertItem(*it, n++);            
        }
}


void TopLevel::recentActivated(int n)
{
    openSandbox( *(recentFiles.at(n)) );
}


void TopLevel::showMenuToolTip(int id)
{
    // Hack, replace by actions
    
    QString text = tooltips[id];
    statusBar()->message(text);
}


void TopLevel::setFilter()
{
    UpdateView::Filter filter = UpdateView::Filter(0);
    if (opt_hideFiles)
        filter = UpdateView::Filter(filter | UpdateView::OnlyDirectories);
    if (opt_hideUpToDate)
        filter = UpdateView::Filter(filter | UpdateView::NoUpToDate);
    if (opt_hideRemoved)
        filter = UpdateView::Filter(filter | UpdateView::NoRemoved);
    update->setFilter(filter);
   
    settings->setItemChecked(settings_hidefiles, opt_hideFiles);
    settings->setItemChecked(settings_hideuptodate, opt_hideUpToDate);
    settings->setItemChecked(settings_hideremoved, opt_hideRemoved);

    QString str;
    if (opt_hideFiles)
        str = "F";
    else
        {
            if (opt_hideUpToDate)
                str += "N";
            if (opt_hideRemoved)
                str += "R";
        }
    
    filterLabel->setText(str);
}


#if 0
void TopLevel::parseStatus(QString pathname, QStrList list)
{
    char buf[512];
    QString command;
    QString dirpath;
    QString name, statusstr, version;
    enum { Begin, File, FileSep, WorkRev, FinalSep } state;

    QString line = joinLine(list);
    
    command = "cd ";
    command += pathname;
    command += " && " + cvsClient() + " status -l ";
    command += line;
    //    command += " 2>&1";

    FILE *f = popen(command, "r");
    if (!f)
        return;

    state = Begin;
    while (fgets(buf, sizeof buf, f))
        {
            QCString line = buf;
            chomp(&line);
            //            DEBUGOUT( "Line: " << line );
            switch (state)
                {
                case Begin:
                    if (line.left(22) == "cvs status: Examining ")
                        dirpath = line.right(line.length()-22);
                    state = File;
                    //                    DEBUGOUT( "state = file" );
                    break;
                case File:
                    if (line.length() > 32 &&
                        line.left(6) == "File: " &&
                        line.mid(24, 8) == "Status: ")
                        {
                            name = line.mid(6, 18).stripWhiteSpace();
                            if (dirpath != ".")
                                name.prepend("/").prepend(dirpath);
                            statusstr = line.right(line.length()-32)
                                .stripWhiteSpace();
                            state = FileSep;
                            //                            DEBUGOUT( "state = FileSep" );
                        }
                    break;
                case FileSep:
                    if (!line.isEmpty()) ; // Error
                    state = WorkRev;
                    //                    DEBUGOUT( "state = WorkRev" );
                    break;
                case WorkRev:
                    if (line.left(21) == "   Working revision:\t")
                        {
                            int pos;
                            version = line.right(line.length()-21);
                            if ( (pos = version.find(" ")) != -1 )
                                version.truncate(pos);
                            state = FinalSep;
                            //                            DEBUGOUT( "state = FinalSep" );
                        }
                    break;
                case FinalSep:
                    if (line == "")
                        {
                            //                            DEBUGOUT( "Adding: " << name <<
                            //                                      "Status: " << statusstr << "Version: " << version );
                            UpdateView::Status status = UpdateView::Unknown;
                            if (statusstr == "Up-to-date")
                                status = UpdateView::UpToDate;
                            else if (statusstr == "Locally Modified")
                                status = UpdateView::LocallyModified;
                            else if (statusstr == "Locally Added")
                                status = UpdateView::LocallyAdded;
                            else if (statusstr == "Locally Removed")
                                status = UpdateView::LocallyRemoved;
                            else if (statusstr == "Needs Checkout")
                                status = UpdateView::NeedsUpdate;
                            else if (statusstr == "Needs Patch")
                                status = UpdateView::NeedsPatch;
                            else if (statusstr == "Needs Merge")
                                status = UpdateView::NeedsMerge;
                            else if (statusstr == "File had conflicts on merge")
                                status = UpdateView::Conflict;
                            //                            update->addEntry(status, name /*, version*/);
                            state = Begin;
                            //                            DEBUGOUT( "state = Begin" );
                        }
                }
        }
    pclose(f);
}
#endif


bool TopLevel::queryExit()
{
    KConfig *config = kapp->config();
    
    config->setGroup("Recent sandboxes");
    for (int i = 1; recentFiles.count(); ++i)
        {
            QString str;
            str.setNum(i);
            str.prepend("Sandbox");
            config->writeEntry(str, *(recentFiles.at(0)));
            recentFiles.remove(recentFiles.begin());
        }
    
    config->setGroup("Main window");
    config->writeEntry("Customized", true);
    config->writeEntry("Size", size());

    config->setGroup("Diff dialog");
    DiffDialog::saveOptions(config);
    config->setGroup("Log dialog");
    LogDialog::saveOptions(config);
    config->setGroup("LogList view");
    LogListView::saveOptions(config);
    config->setGroup("Resolve dialog");
    ResolveDialog::saveOptions(config);
    config->setGroup("Resolve edit dialog");
    ResolveEditorDialog::saveOptions(config);
    config->setGroup("Commit dialog");
    CommitDialog::saveOptions(config);
    config->setGroup("ChangeLog dialog");
    ChangeLogDialog::saveOptions(config);
    config->setGroup("Annotate dialog");
    AnnotateDialog::saveOptions(config);
    config->setGroup("Checkout dialog");
    CheckoutDialog::saveOptions(config);
    config->setGroup("History dialog");
    HistoryDialog::saveOptions(config);
    config->setGroup("Repository dialog");
    RepositoryDialog::saveOptions(config);
    config->setGroup("AddRepository dialog");
    AddRepositoryDialog::saveOptions(config);

    config->setGroup("Session");
    saveProperties(config);

    config->sync();
    return true;
}


void TopLevel::restorePseudo(const QString &dirname)
{
    KConfig *config = kapp->config();

    config->setGroup("Recent sandboxes");
    for (int i = 1; ; ++i)
        {
            QString str("Sandbox");
            str += QString::number(i);
            QString entry = config->readEntry(str);
            if (entry.isEmpty())
                break;
            
            recentFiles.append(entry);
        }
    
    config->setGroup("Main window");
    if (config->readEntry("Customized"))
        resize(config->readSizeEntry("Size"));

    config->setGroup("Diff dialog");
    DiffDialog::loadOptions(config);
    config->setGroup("Log dialog");
    LogDialog::loadOptions(config);
    config->setGroup("LogList view");
    LogListView::loadOptions(config);
    config->setGroup("Resolve dialog");
    ResolveDialog::loadOptions(config);
    config->setGroup("Resolve edit dialog");
    ResolveEditorDialog::loadOptions(config);
    config->setGroup("Commmit dialog");
    CommitDialog::loadOptions(config);
    config->setGroup("ChangeLog dialog");
    ChangeLogDialog::loadOptions(config);
    config->setGroup("Annotate dialog");
    AnnotateDialog::loadOptions(config);
    config->setGroup("Checkout dialog");
    CheckoutDialog::loadOptions(config);
    config->setGroup("History dialog");
    HistoryDialog::loadOptions(config);
    config->setGroup("Repository dialog");
    RepositoryDialog::loadOptions(config);
    config->setGroup("AddRepository dialog");
    AddRepositoryDialog::loadOptions(config);

    config->setGroup("Session");
    if (!dirname.isEmpty())
        config->writeEntry("Current Directory", dirname);
    readProperties(config);
}

 
void TopLevel::readProperties(KConfig *config)
{
    // Unfortunately, the KConfig systems sucks and we have to live
    // with all entries in one group for session management.
    
    opt_createDirs = config->readBoolEntry("Create Dirs", true);
    settings->setItemChecked(settings_create, opt_createDirs);
    
    opt_pruneDirs = config->readBoolEntry("Prune Dirs", true);
    settings->setItemChecked(settings_prune, opt_pruneDirs);
    
    opt_updateRecursive = config->readBoolEntry("Update Recursive", false);
    settings->setItemChecked(settings_updaterecur, opt_updateRecursive);

    opt_commitRecursive = config->readBoolEntry("Commit Recursive", false);
    settings->setItemChecked(settings_commitrecur, opt_commitRecursive);

    opt_doCVSEdit = config->readBoolEntry("Do cvs edit", false);
    settings->setItemChecked(settings_docvsedit, opt_doCVSEdit);

    opt_hideFiles = config->readBoolEntry("Hide Files", false);
    opt_hideUpToDate = config->readBoolEntry("Hide UpToDate Files", false);
    opt_hideRemoved = config->readBoolEntry("Hide Removed Files", false);
    setFilter();

    int splitterpos1 = config->readNumEntry("Splitter Pos 1", 0);
    int splitterpos2 = config->readNumEntry("Splitter Pos 2", 0);
    if (splitterpos1)
        {
            QValueList<int> sizes;
            sizes << splitterpos1;
            sizes << splitterpos2;
            splitter->setSizes(sizes);
        }

    QString currentDir = config->readEntry("Current Directory");
    if (!currentDir.isEmpty())
        openSandbox(currentDir);
}


void TopLevel::saveProperties(KConfig *config)
{
    config->writeEntry("Create Dirs", opt_createDirs);
    config->writeEntry("Prune Dirs", opt_pruneDirs);
    config->writeEntry("Update Recursive", opt_updateRecursive);
    config->writeEntry("Commit Recursive", opt_commitRecursive);
    config->writeEntry("Do cvs edit", opt_doCVSEdit);
    config->writeEntry("Hide Files", opt_hideFiles);
    config->writeEntry("Hide UpToDate Files", opt_hideUpToDate);
    config->writeEntry("Hide Removed Files", opt_hideRemoved);
    QValueList<int> sizes = splitter->sizes();
    config->writeEntry("Splitter Pos 1", sizes[0]);
    config->writeEntry("Splitter Pos 2", sizes[1]);
    config->writeEntry("Current Directory", sandbox);
}


// Local Variables:
// c-basic-offset: 4
// End:
